use std::{str::FromStr, sync::Arc};

use axum::{
    extract::Request,
    http::{self, HeaderMap, StatusCode},
    middleware::Next,
    response::IntoResponse,
};
use protocol::{SignalType, User};
use serde::{Deserialize, Serialize};

#[derive(Debug, Serialize, Deserialize)]
pub struct UserAndExpectedSignal {
    pub user: User,
    pub expected_signal: SignalType,
}

async fn extract_user_from_authorization_header(headers: &HeaderMap) -> Option<User> {
    if let Some(authorization_header) = headers.get(http::header::AUTHORIZATION) {
        dbg!(authorization_header);
        if let Ok(authorization_header) = authorization_header.to_str() {
            if let Some((bearer, authorization_token)) = authorization_header.split_once(' ') {
                println!(
                    "Info: Extraction | Authorization Header | {} || {}",
                    bearer, authorization_token
                );
                if bearer.to_lowercase() == "bearer" {
                    let user = User {
                        username: authorization_token.to_string(),
                    };
                    return Some(user);
                }
            }
        }
    }
    None
}

pub async fn verify_then_get_user(mut request: Request, next: Next) -> impl IntoResponse {
    let headers = request.headers();
    println!("Info: Verify | Headers| {:#?}", headers);
    if let Some(user) = extract_user_from_authorization_header(headers).await {
        let user = Arc::new(user);
        request.extensions_mut().insert(user);
        return next.run(request).await;
    }

    StatusCode::FORBIDDEN.into_response()
}

pub async fn verify_then_get_user_and_expected_signal(
    mut request: Request,
    next: Next,
) -> impl IntoResponse {
    let headers = request.headers();
    if let Some(user) = extract_user_from_authorization_header(headers).await {
        if let Ok(expected_signal) = headers.get("EXPECTED_SIGNAL").unwrap().to_str() {
            match SignalType::from_str(expected_signal) {
                Ok(expected_signal) => {
                    let user_and_expected_signal = UserAndExpectedSignal {
                        user,
                        expected_signal,
                    };
                    let user_and_expected_signal = Arc::new(user_and_expected_signal);
                    request.extensions_mut().insert(user_and_expected_signal);
                    next.run(request).await
                }
                Err(err_val) => {
                    eprintln!(
                        "Error: Verify and Get Expected Signal | Signal Type Conversion | {}",
                        err_val
                    );
                    StatusCode::BAD_REQUEST.into_response()
                }
            }
        } else {
            StatusCode::BAD_REQUEST.into_response()
        }
    } else {
        StatusCode::FORBIDDEN.into_response()
    }
}
